require 'sketchup.rb'

#-----------------------------------------------------------------------------

class FPS
  def get_view_info
    @@view = Sketchup.active_model.active_view
    camera = @@view.camera
    @@target = camera.target
    @@up = [0,0,1]
    @@eye = camera.eye
    @direction = camera.direction
    @direction.z = 0
    @xaxis = camera.xaxis
    @centery = @@view.vpheight/2
    @centerx = @@view.vpwidth/2
  end#get_view_info
  
  def jump
    u = 150 #initial velocity in inches per second
    t = 0.0 #time in seconds
    a = -384 #aceleration in inches per second squared
    h = 0 #height in inches
    while (h >= 0 and t < 5)
      h = u*t + 0.5*a*(t**2)
      puts "time: " + t.to_s + "   height: " + h.to_s + "   speed: " + v.to_s
      t = t += 0.01
      v = u + a*t
    end#if
  end#jump
  
  def reset(view)
    @@step = Geom::Point3d.new 0,0,0
    @@look = Geom::Point3d.new 0,0,0
    @@jump = 0
    @@eye.z = 66.0
    @stepsize = 6
    @@view.camera.set(@@eye,@@target,@@up)
    @x1 = @y1 = nil
    view.invalidate
    puts "FPS has been reset." 
  end
  
  def activate
    self.get_view_info
    self.reset(@@view)
    puts "FPS has been activated."
  end#activate
  
  def onMouseMove(flags, x, y, view)
    self.get_view_info
    #puts "x:   pixels = " + x.to_s 
    #puts "y:   pixels = " + y.to_s
    if (@x1 == nil and @y1 == nil)
      @x1 = x
      @y1 = y
      sleep (0.002)
    else
      x2 = x 
      y2 = y 
      turnLR = (x2-@x1)/@centerx.to_f * -180
      turnUD = (y2-@y1)/@centery.to_f * -90
      rotateLR = Geom::Transformation.rotation @@eye, @@up, turnLR.degrees
      rotateUD = Geom::Transformation.rotation @@eye, @xaxis, turnUD.degrees
      new_target = @@target.transform rotateLR
      new_target = new_target.transform rotateUD
      @@look = new_target.- @@target
      @x1 = @y1 = nil
    end
  end#onMouseMove
    
  def onKeyDown(key, repeat, flags, view)
    #puts "onKeyDown:   key = " + key.to_s 
    self.get_view_info
    if( key == VK_UP)
      @@step.offset! @direction,@stepsize
      @vk_up = true
    end#if
    if( key == VK_DOWN)
      @@step.offset! @direction,-@stepsize
      @vk_down = true
    end#if
    if( key == VK_RIGHT)
      @@step.offset! @xaxis,@stepsize
      @vk_right = true
    end#if
    if( key == VK_LEFT)
      @@step.offset! @xaxis,-@stepsize
      @vk_left = true
    end#if
    if( key == VK_CONTROL)
      @@t = 0.0 #time in seconds
      @@h2 = 0
      @@jump = 150 #initial velocity in inches per second
      @vk_control = true
    end#if
    puts "step: " + @@step.to_s 
    @@view.animation = FPS.new
  end#onKeyDown
  
  def onKeyUp(key, repeat, flags, view)
    #puts "onKeyUp:   key = " + key.to_s 
    if( key == VK_UP)
      @@step.offset! @direction,-@stepsize
      @vk_up = false
    end#if
    if( key == VK_DOWN)
      @@step.offset! @direction,@stepsize
      @vk_down = false
    end#if
    if( key == VK_RIGHT)
      @@step.offset! @xaxis,-@stepsize
      @vk_right = false
    end#if 
    if( key == VK_LEFT)
      @@step.offset! @xaxis,@stepsize
      @vk_left = false
    end#if
    if( key == VK_CONTROL)
      @vk_control = false
      #@@jump = 0
    end#if
    if (@vk_up == false and @vk_down == false and @vk_right == false and @vk_left == false)
      @@step = Geom::Point3d.new 0,0,0
    end
    #puts "step: " + @@step.to_s 
  end
  
  # onCancel is called when the user hits the escape key
  def onCancel(flag, view)
    puts "FPS has been canceled."
    self.reset(view)
  end
  
  # deactivate is called when the tool is deactivated because
  # a different tool was selected
  def deactivate(view)
    view.invalidate
    puts "FPS has been deactivated." 
  end
  
  def nextFrame(view)
    if @@jump > 0
      h1 = @@jump*@@t + 0.5*-384*(@@t**2)
      @@h2 = @@jump = 0 if h1 < 0
      h = h1 - @@h2
      #puts "time: " + @@t.to_s + "   displace: " + h.to_s
      @@eye.z = @@eye.z + h
      @@t = @@t += 0.05
      @@h2 = h1
    end
    
    @@eye = @@eye.+ @@step.to_a
    @@target = @@target.+ @@step.to_a
    @@target = @@target.+ @@look.to_a
    @@view.camera.set(@@eye,@@target,@@up)
    view.show_frame
    self.get_view_info
    @@look = Geom::Point3d.new 0,0,0
    return true
  end#nextFrame
  
end # class FPS

# This functions is just a shortcut for selecting the new tool
UI.menu("Camera").add_item("FPS - First Person 'Sketchuper'") {
  Sketchup.active_model.select_tool FPS.new
}
